#include <stdio.h>
#include <gb/gb.h>
#include <gb/console.h>

#define GAME_BOARD_CELL_EMPTY     ' '
#define GAME_BOARD_CELL_PLAYER    'x'
#define GAME_BOARD_CELL_COMPUTER  'o'

UINT8 GAME_BOARD[9];  // (3×3 cases)

void clear_screen(void) {
    UINT8 x;
    UINT8 y = 18;
    while (y) {
        y -= 1;
        x = 20;
        while (x) {
            x -= 1;
            gotoxy(x, y);
            setchar(' ');
        }
    }
}

UINT8 coord_2d_to_1d(UINT8 x, UINT8 y) {
    return y * 3 + x;
}

void title_screen(void) {
    // 1. Effacer tout ce qui se trouve à l'écran
    clear_screen();
    // 2. Afficher notre écran de titre
    gotoxy(4, 5);
    printf("Tic Tac Toe");
    gotoxy(3, 15);
    printf("- Press START -");
    // 3. Attendre que le joueur appuie sur START et qu'il relâche le bouton
    waitpad(J_START);
    waitpadup();
}

void game_init(void) {
    UINT8 i;

    for (i = 0 ; i < 9 ; i += 1) {
        GAME_BOARD[i] = GAME_BOARD_CELL_EMPTY;
    }
}

void game_draw_board(void) {
    // 1. On efface l'écran
    clear_screen();
    // 2. On « dessine » le plateau
    gotoxy(4, 1);
    printf("Tic Tac Toe");
    gotoxy(1, 17);
    printf("X You - O Computer");
    gotoxy(0, 4);
    printf("       |   |   \n");
    printf("       |   |   \n");
    printf("       |   |   \n");
    printf("    ---+---+---\n");
    printf("       |   |   \n");
    printf("       |   |   \n");
    printf("       |   |   \n");
    printf("    ---+---+---\n");
    printf("       |   |   \n");
    printf("       |   |   \n");
    printf("       |   |   \n");
}

void game_draw_state(void) {
    UINT8 i;
    UINT8 x;
    UINT8 y;
    UINT8 graph_x;
    UINT8 graph_y;

    for (y = 0 ; y < 3 ; y += 1) {
        for (x = 0 ; x < 3 ; x += 1) {
            i = coord_2d_to_1d(x, y);
            graph_x = 4 + x * 4 + 1;
            graph_y = 4 + y * 4 + 1;
            gotoxy(graph_x, graph_y);
            setchar(GAME_BOARD[i]);
        }
    }

}

void game(void) {
    game_init();
    game_draw_board();

    // On rajoute des pions sur le plateau et on appelle notre fonction
    // game_draw_state() pour tester que tout fonctionne bien
    GAME_BOARD[0] = GAME_BOARD_CELL_COMPUTER;
    GAME_BOARD[4] = GAME_BOARD_CELL_PLAYER;
    game_draw_state();
}

void main(void) {
    // Boucle principale du programme qui alternera à terme entre affichage de
    // l'écran de titre et la phase de jeu.
    while (1) {
        title_screen();
        game();

        // On bloque temporairement le programme à cette étape histoire
        // d'avoir le temps d'admirer le résultat. On supprimera ça par
        // la suite.
        waitpad(J_START);
        waitpadup();
    }
}
